#ifndef __MODEL_H
#define __MODEL_H

#include "token.h"

enum expression_type
{
	EXPR_INT,
	EXPR_FLOAT,
	EXPR_BINOP,
	EXPR_UNOP,
	EXPR_GROUP,

	/* TODO: Statements */
};

struct expression
{
	enum expression_type type;

	union
	{
		/* integer (i.e. 2) */
		int integer;

		/* floating (i.e. 3.141592) */
		float floating;

		struct
		{
			/* binary operations (i.e. x + y) */
			struct token *op;
			struct expression *left;
			struct expression *right;
		} binop;

		struct
		{
			/* unary operations (i.e. -x) */
			struct token *op;
			struct expression *operand;
		} unop;

		struct
		{
			/* grouping: ( <expr> ) */
			struct expression *value;
		} grouping;
	};

	/* TODO: Statements */
};
struct expression *expression_create_integer (int value);
struct expression *expression_create_floating (float value);
struct expression *expression_create_binop (struct token *op,
																						struct expression *left,
																						struct expression *right);
struct expression *expression_create_unop (struct token *op,
																					 struct expression *operand);
struct expression *expression_create_grouping (struct expression *value);

#endif
